<?php
/**
 * BuddyBlog Pro Form Custom fields ajax handler.
 *
 * @package    BuddyBlog_Pro
 * @subpackage Admin
 * @copyright  Copyright (c) 2020, Brajesh Singh
 * @license    https://www.gnu.org/licenses/gpl.html GNU Public License
 * @author     Brajesh Singh
 * @since      1.0.0
 */

namespace BuddyBlog_Pro\Admin;

// Do not allow direct access over web.
defined( 'ABSPATH' ) || exit;

/**
 * Handle form actions via ajax for managing custom fields.
 */
class Form_Admin_Custom_Fields_Ajax_Handler {

	/**
	 * Setup the bootstrapper.
	 */
	public static function boot() {
		$self = new self();
		$self->setup();
	}

	/**
	 * Register actions.
	 */
	private function setup() {
		add_action( 'wp_ajax_bblogpro_form_add_custom_field', array( $this, 'add_field' ) );
		add_action( 'wp_ajax_bblogpro_form_sort_custom_fields', array( $this, 'sort_fields' ) );
		add_action( 'wp_ajax_bblogpro_form_delete_custom_field', array( $this, 'delete_field' ) );
		add_filter( 'bblpro_custom_field_settings', array( $this, 'add_extra' ), 10, 3 );
	}

	/**
	 * Add core field.
	 */
	public function add_field() {

		if ( ! wp_verify_nonce( wp_unslash( $_POST['_wpnonce'] ), 'bblogpro_add_form' ) || ! bblpro_user_can_manage_forms( get_current_user_id() ) ) {
			wp_send_json_error(
				array(
					'message' => __( 'Invalid Request.', 'buddyblog-pro' ),
					'fields'  => array(),
				)
			);
		}

		// validate fields.
		$form_id = isset( $_POST['form_id'] ) ? absint( $_POST['form_id'] ) : 0;
		$form    = get_post( $form_id );

		if ( empty( $form ) || bblpro_get_form_post_type() !== $form->post_type ) {
			wp_send_json_error(
				array(
					'message' => __( 'Invalid Request.', 'buddyblog-pro' ),
					'fields'  => array(),
				)
			);
		}

		$errors          = array();
		$required_fields = apply_filters(
			'bblpro_custom_field_settings_required_fields',
			array(
				'type',
				'label',
			)
		);

		foreach ( $required_fields as $field_name ) {
			if ( empty( $_POST[ $field_name ] ) ) {
				$errors[ $field_name ] = sprintf( '%s is required.', $field_name );
			}
		}

		$errors = apply_filters( 'bblpro_custom_field_settings_validate', $errors, $form );

		if ( ! empty( $errors ) ) {
			wp_send_json_error(
				array(
					'message' => __( 'Required field missing.', 'buddyblog-pro' ),
					'fields'  => $errors,
				)
			);
		}

		// if we are here, let us update the settings.
		$settings = get_post_meta( $form_id, '_buddyblog_custom_fields', true );

		if ( empty( $settings ) ) {
			$settings = array();
		}

		$key = ! empty( $_POST['key'] ) ? sanitize_key( wp_unslash( $_POST['key'] ) ) : sanitize_key( wp_unslash( $_POST['label'] ) );

		if ( isset( $settings[ $key ] ) ) {
			// Can not add the field twice.
			wp_send_json_error(
				array(
					'message' => __( 'Can not add the field with same key twice.', 'buddyblog-pro' ),
					'fields'  => array( $key ),
				)
			);
		}

		$type = isset( $_POST['type'] ) ? wp_unslash( $_POST['type'] ) : '';
		// add field to the array.
		$settings[ $key ] = apply_filters(
			'bblpro_custom_field_settings',
			array(
				'type'        => $type,
				'key'         => $key,
				'label'       => isset( $_POST['label'] ) ? esc_html( wp_unslash( $_POST['label'] ) ) : '',
				'is_required' => empty( $_POST['is_required'] ) ? 0 : 1,
				'default'     => isset( $_POST['default_value'] ) ? wp_unslash( $_POST['default_value'] ) : '',
				'placeholder' => isset( $_POST['placeholder'] ) ? wp_strip_all_tags( wp_unslash( $_POST['placeholder'] ) ) : '',
			),
			$type,
			$form
		);

		$settings = array_filter( $settings );
		update_post_meta( $form_id, '_buddyblog_custom_fields', $settings );

		$this->send_table( $form_id, __( 'Saved.', 'buddyblog-pro' ) );
	}

	/**
	 * Sends core fields list.
	 *
	 * @param int    $form_id form id.
	 * @param string $message message.
	 */
	private function send_table( $form_id, $message ) {

		ob_start();

		bblpro_print_custom_fields_table( $form_id );
		$content = ob_get_clean();

		wp_send_json_success(
			array(
				'content' => $content,
				'message' => $message,
			)
		);
	}

	/**
	 * Sort fields.
	 */
	public function sort_fields() {

		if ( ! wp_verify_nonce( wp_unslash( $_POST['_wpnonce'] ), 'bblogpro_add_form' ) || ! bblpro_user_can_manage_forms( get_current_user_id() ) ) {
			wp_send_json_error(
				array(
					'message' => __( 'Invalid Request.', 'buddyblog-pro' ),
					'fields'  => array(),
				)
			);
		}

		$fields = isset( $_POST['fields'] ) ? array_map( 'wp_unslash', $_POST['fields'] ) : array();
		$form_id = isset( $_POST['form_id'] ) ? absint( $_POST['form_id'] ) : 0;

		$form = get_post( $form_id );

		if ( empty( $fields ) || empty( $form ) || bblpro_get_form_post_type() !== $form->post_type ) {
			wp_send_json_error(
				array(
					'message' => __( 'Invalid Request.', 'buddyblog-pro' ),
					'fields'  => array(),
				)
			);
		}

		// if we are here, let us update the settings.
		$settings = get_post_meta( $form_id, '_buddyblog_custom_fields', true );

		if ( empty( $settings ) ) {
			$settings = array();
		}

		$sorted_fields = array();

		$field_types = bblpro_get_custom_field_types();

		foreach ( $fields as $field ) {

			if ( ! isset( $settings[ $field ] ) ) {
				continue;
			}

			$type = isset( $settings[ $field ]['type'] ) ? $settings[ $field ]['type'] : '';

			if ( ! $type || ! isset( $field_types[ $type ] ) ) {
				continue;
			}
			$sorted_fields[ $field ] = $settings[ $field ];
		}

		if ( empty( $sorted_fields ) ) {
			// Can not add the field twice.
			wp_send_json_error(
				array(
					'message' => __( 'Unable to update.', 'buddyblog-pro' ),
					'fields'  => array(),
				)
			);
		}

		update_post_meta( $form_id, '_buddyblog_custom_fields', $sorted_fields );

		$this->send_table( $form_id, __( 'Updated.', 'buddyblog-pro' ) );
	}

	/**
	 * Add core field.
	 */
	public function delete_field() {

		if ( ! wp_verify_nonce( wp_unslash( $_POST['_wpnonce'] ), 'bblogpro_add_form' ) || ! bblpro_user_can_manage_forms( get_current_user_id() ) ) {
			wp_send_json_error(
				array(
					'message' => __( 'Invalid Request.', 'buddyblog-pro' ),
					'fields'  => array(),
				)
			);
		}

		$field_key = isset( $_POST['field'] ) ? trim( wp_unslash( $_POST['field'] ) ) : '';
		$form_id   = isset( $_POST['form_id'] ) ? absint( $_POST['form_id'] ) : 0;

		$form = get_post( $form_id );

		if ( empty( $field_key ) || empty( $form ) || bblpro_get_form_post_type() !== $form->post_type ) {
			wp_send_json_error(
				array(
					'message' => __( 'Invalid Request.', 'buddyblog-pro' ),
					'fields'  => array(),
				)
			);
		}

		// if we are here, let us update the settings.
		$settings = get_post_meta( $form_id, '_buddyblog_custom_fields', true );

		if ( empty( $settings ) ) {
			$settings = array();
		}

		if ( ! isset( $settings[ $field_key ] ) ) {
			// Can not add the field twice.
			wp_send_json_error(
				array(
					'message' => __( 'Can not remove the field.', 'buddyblog-pro' ),
					'fields'  => array( $field_key ),
				)
			);
		}

		// add field to the array.
		unset( $settings[ $field_key ] );
		$settings = array_filter( $settings );
		update_post_meta( $form_id, '_buddyblog_custom_fields', $settings );

		$this->send_table( $form_id, __( 'Deleted.', 'buddyblog-pro' ) );
	}

	/**
	 * Add extra options.
	 *
	 * @param array    $data field settings.
	 * @param string   $type field type.
	 * @param \WP_Post $post Post object.
	 *
	 * @return mixed
	 */
	public function add_extra( $data, $type, $post ) {

		if ( ! in_array( $type, array( 'select', 'multiselect', 'radio', 'checkbox' ), true ) ) {
			return $data;
		}

		$option_key       = 'bbl-input-cf-field-' . $type . '-options';
		$option_value_key = 'bbl-input-cf-field-' . $type . '-labels';

		// @todo add Sanitization for single/multi value in future.
		$data['field-options-keys']   = isset( $_POST[ $option_key ] ) ? wp_unslash( $_POST[ $option_key ] ) : '';
		$data['field-options-labels'] = isset( $_POST[ $option_value_key ] ) ? wp_unslash( $_POST[ $option_value_key ] ) : '';

		return $data;
	}
}
