<?php
/**
 * Filed type functions
 *
 * @package    BuddyBlog_Pro
 * @subpackage Core
 * @copyright  Copyright (c) 2020, Brajesh Singh
 * @license    https://www.gnu.org/licenses/gpl.html GNU Public License
 * @author     Brajesh Singh
 * @since      1.0.0
 */

// Do not allow direct access over web.
defined( 'ABSPATH' ) || exit;

/**
 * Get available core fields.
 *
 * @return array
 */
function bblpro_get_core_field_types() {

	return apply_filters(
		'bblpro_core_field_types',
		array(
			'post_title'   => _x( 'Post Title', 'Core post field label', 'buddyblog-pro' ),
			'post_content' => _x( 'Post Content', 'Core post field label', 'buddyblog-pro' ),
			'thumbnail'    => _x( 'Featured Image', 'Core post field label', 'buddyblog-pro' ),
			// 'excerpt'        => __( 'Excerpt', 'buddyblog-pro' ),
		)
	);
}

/**
 * Get available custom fields.
 *
 * @return array
 */
function bblpro_get_custom_field_types() {

	return apply_filters(
		'bblpro_custom_field_types',
		array(
			'text'        => _x( 'Text', 'Field type label', 'buddyblog-pro' ),
			'textarea'    => _x( 'Text Area', 'Field type label', 'buddyblog-pro' ),
			'number'      => _x( 'Number', 'Field type label', 'buddyblog-pro' ),
			'url'         => _x( 'Web URL', 'Field type label', 'buddyblog-pro' ),
			//'image'       => __( 'Image', 'buddyblog-pro' ),
			//'file'        => __( 'File', 'buddyblog-pro' ),
			'select'      => _x( 'Select', 'Field type label', 'buddyblog-pro' ),
			'multiselect' => _x( 'Multi Select', 'Field type label', 'buddyblog-pro' ),
			'radio'       => _x( 'Radio', 'Field type label', 'buddyblog-pro' ),
			'checkbox'    => _x( 'Checkbox', 'Field type label', 'buddyblog-pro' ),
		)
	);
}

/**
 * Get the label for the core post field.
 *
 * @param string $filed_type type.
 *
 * @return string
 */
function bblpro_get_core_field_label( $filed_type ) {

	$fields = bblpro_get_core_field_types();

	return isset( $fields[ $filed_type ] ) ? $fields[ $filed_type ] : _x( 'Invalid Type.', 'Field type label', 'buddyblog-pro' );
}

/**
 * Get th label for the custom post field.
 *
 * @param string $field_type type.
 *
 * @return string
 */
function bblpro_get_custom_field_label( $field_type ) {

	$fields = bblpro_get_custom_field_types();

	return isset( $fields[ $field_type ] ) ? $fields[ $field_type ] : _x( 'Invalid Type.', 'Field type label', 'buddyblog-pro' );
}

/**
 * Prepare options as key/value pair.
 *
 * @param array $field_settings settings array.
 *
 * @return array
 */
function bblpro_prepare_meta_options( $field_settings ) {

	if ( empty( $field_settings['field-options-keys'] ) || empty( $field_settings['field-options-labels'] ) ) {
		return array();
	}

	$values  = explode( ',', $field_settings['field-options-keys'] );
	$labels  = explode( ',', $field_settings['field-options-labels'] );
	$options = array();
	$count   = count( $values ) < count( $labels ) ? count( $values ) : count( $labels );

	for ( $i = 0; $i < $count; $i ++ ) {
		$options[] = array(
			'value' => trim( $values[ $i ] ),
			'label' => trim( $labels[ $i ] ),
		);
	}

	return $options;
}

/**
 * Sanitize value.
 *
 * @param string $field_type field type.
 * @param mixed  $value value.
 * @param array  $field_settings field settings.
 *
 * @return mixed|null
 */
function bblpro_get_sanitized_value( $field_type, $value, $field_settings ) {

	$sanitized = '';
	$options   = bblpro_prepare_meta_options( $field_settings );
	// value was not specified.
	// we should use default if given else return null.
	if ( is_null( $value ) ) {
		if ( isset( $field_settings['default'] ) && '' !== $field_settings['default'] ) {
			$value = $field_settings['default'];
		} else {
			return null;
		}
	}

	switch ( $field_type ) {
		case 'textbox':
		case 'text':
		case 'date':
		case 'textarea':
		case 'hidden':
			$sanitized = esc_attr( $value ); // should we escape?
			break;

		case 'radio':
		case 'select':
			foreach ( $options as $option ) {
				if ( $option['value'] == $value ) { // the loose comaprision is intentional.
					$sanitized = $value;
				}
			}

			break;
		// for checkbox.
		case 'checkbox':
		case 'multiselect':
			$vals = array();

			foreach ( $options as $option ) {// how to validate?
				$vals[] = $option['value'];
			}
			$sanitized = array_intersect( $vals, (array) $value );
			break;

		case 'url':
			$sanitized = esc_url( $value );
			break;

		case 'number':
			$sanitized = intval( $value );
			break;
		default:
			$sanitized = null;
			break;
	}

	return apply_filters( 'bblpro_sanitized_value', $sanitized, $value, $field_type, $field_settings );
}
